
from distutils.core import setup
setup(
  name = 'parski',
  packages = ['parski'],
  version = '1.1.2.42',
  description = 'Helper library for parsing vpc data feed',
  author = 'Russell Endicott',
  author_email = 'rendicott@gmail.com',
  url = 'https://github.build.company.com/CloudPod/parski', # use the URL to the github repo
  download_url = 'https://github.build.company.com/CloudPod/parski/releases/download/1.1.2.42/parski-1.1.2.42.tar.gz', # I'll explain this in a second
  keywords = ['vpc', 'json', 'parski'],
  classifiers = [],
)


class TableFormat:
    """
    Class helper for building tableformat outputs.
    """

    def __init__(self, l_attributes, l_values, dict_extend_values=None, l_ignore_attrs=None, html=None):
        if html is None:
            html = False
        self.html = html
        self.dict_extend_values = dict_extend_values
        self.ignore_attrs = l_ignore_attrs
        (self.attributes, self.values_raw) = self.cull_ignored(l_attributes, l_values)
        self.attributes_lengths = []
        self.values_trunc = []
        if self.html:
            self.fmt_shell = '<td width="{1}">{{{0}}}</td>'
        else:
            self.fmt_shell = '{{{0}:{1}}}'
        self.fmt = self.gen_fmt()
        self.truncate_values()
        self.string_values = self.gen_string()
        self.string_header = self.gen_header()

    def cull_ignored(self, attributes, values):
        scope = len(attributes)
        topop = []
        for i in range(0, scope):
            w_attr = attributes[i]
            if w_attr in self.ignore_attrs:
                topop.append(i)
        new_attr = []
        new_values = []
        for i in range(0, scope):
            if i not in topop:
                new_attr.append(attributes[i])
                new_values.append(values[i])
        return new_attr, new_values

    def gen_fmt(self):
        """
        Generates the format string (e.g., {0:4}{1:15})
        based on the length of the attribute names
        :return: fmt_string
        """
        fmt_string = ''
        for attr in self.attributes:
            self.attributes_lengths.append(len(attr))
        width = len(self.attributes)
        modifier = 1
        if self.html:
            modifier = 12
        for i in range(0, width):
            try:
                field_length = (self.dict_extend_values[self.attributes[i]]) * modifier
            except:
                field_length = (self.attributes_lengths[i] + 2) * modifier
            fmt_string += self.fmt_shell.format(str(i), str(field_length))
        return fmt_string

    def truncate_values(self):
        """
        Truncates the values a little smaller than the column widths
        :return:
        """
        scope = len(self.attributes)
        for i in range(0, scope):
            try:
                width = self.dict_extend_values[self.attributes[i]]
            except:
                width = self.attributes_lengths[i]
            value_raw = str(self.values_raw[i])
            try:
                # try subtracting a buffer between columns
                value_trunc = value_raw[:width-1]
            except:
                value_trunc = value_raw[:width]
            self.values_trunc.append(value_trunc)

    def gen_string(self):
        """
        Generates the string of formatted values.
        :return: format_string
        """
        format_string = self.fmt.format(*self.values_trunc)
        if not self.html:
            format_string += '\n'
        return format_string

    def gen_header(self):
        """
        Generates the header using the attribute names formatted by column
        :return: header
        """
        header = self.fmt.format(*self.attributes)
        if not self.html:
            header += '\n'
        return header

class Table_Formattable_Object():
    dump_ignore_attrs = [] #  add strings here of property objs to ignore in dump
    dict_extend_values = {} # add dict strings here of property objs to expand width
    # e.g., {'name': 50, 'age': 5, 'gender': 2}
    def dumpself_tableformat_header(self, html=None):
        if html is None:
            html = False
        msg = ''
        attributes = []
        values = []
        for attr in dir(self):
            if (
                    '__' not in attr and
                    'instancemethod' not in str(type(getattr(self, attr))) and
                    'dict' not in str(type(getattr(self, attr))) and
                    'dump_ignore_attrs' not in attr):
                attributes.append(attr)
                values.append(str(getattr(self, attr)))
        format_helper = TableFormat(attributes,
                                    values,
                                    self.dict_extend_values,
                                    self.dump_ignore_attrs,
                                    html)
        msg += format_helper.string_header
        return msg

    def dumpself_tableformat(self, html=None):
        if html is None:
            html = False
        msg = ''
        attributes = []
        values = []
        for attr in dir(self):
            if (
                    '__' not in attr and
                    'instancemethod' not in str(type(getattr(self, attr))) and
                    'dict' not in str(type(getattr(self, attr))) and
                    'dump_ignore_attrs' not in attr):
                attributes.append(attr)
                values.append(str(getattr(self, attr)))
        format_helper = TableFormat(attributes,
                                    values,
                                    self.dict_extend_values,
                                    self.dump_ignore_attrs,
                                    html)
        msg += format_helper.string_values
        return msg
    def dumpself_csv_header(self):
        msg = ''
        attributes = []
        values = []
        for attr in dir(self):
            if (
                    '__' not in attr and
                    'instancemethod' not in str(type(getattr(self, attr))) and
                    'dict' not in str(type(getattr(self, attr))) and
                    'dump_ignore_attrs' not in attr):
                attributes.append(attr)
                values.append(str(getattr(self, attr)))
                msg += attr + ','
        msg += '\n'
        return msg
    def dumpself_csv(self):
        msg = ''
        attributes = []
        values = []
        for attr in dir(self):
            if (
                    '__' not in attr and
                    'instancemethod' not in str(type(getattr(self, attr))) and
                    'dict' not in str(type(getattr(self, attr))) and
                    'dump_ignore_attrs' not in attr):
                attributes.append(attr)
                values.append(str(getattr(self, attr)))
                msg += '"' + str(getattr(self, attr)) + '"' + ','
        msg += '\n'
        return msg



"""
Takes input json and massages it to make sconbld json (test edit)
"""
from __future__ import print_function

import sys
import os
import time
import json
import re
# import datetime
from copy import deepcopy
import requests

DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%SZ"

def dict_to_paths(search_list, current_path, paths):
    '''
    Convert search_dict to list of paths.
    First call should be dict_to_paths(search_dict, [], [])
    '''
    if isinstance(search_list, dict):
        for key, value in search_list.iteritems():
            temp_path = deepcopy(current_path)
            temp_path.append(key)
            paths = dict_to_paths(search_list[key], temp_path, paths)
    elif isinstance(search_list, list):
        for i, item in enumerate(search_list):
            temp_path = deepcopy(current_path)
            temp_path.append(i)
            paths = dict_to_paths(search_list[i], temp_path, paths)
    else:
        paths.append({'path': current_path, 'value': str(search_list)})
    return paths

def paths_to_dict(input_list):
    #rewrite me to look like dict_to_paths, papa
    #shut up, boy
    search_dict = {}
    for item in input_list:
        sub_dict = search_dict
        for i, step in enumerate(item['path']):
            #check if last step and set value

            if isinstance(step, int):
                #if array extend array until it is the length of step
                while len(sub_dict) <= step:
                    sub_dict.append(None)
            elif isinstance(step, basestring):
                if step not in sub_dict:
                    sub_dict[step] = None

            if i+1 >= len(item['path']):
                sub_dict[step] = item['value']
            #else
            else:
                next_step = item['path'][i+1]
                if isinstance(next_step, int) and sub_dict[step] is None:
                    sub_dict[step] = []
                elif isinstance(next_step, basestring) and sub_dict[step] is None:
                    sub_dict[step] = {}
                sub_dict = sub_dict[step]
    return search_dict

def single_filter(entry, path_list, first):
    '''
    filter single entry with path_list
    '''
    path_groups = []
    for path in path_list:
        #check if path has ended.  If so, remove from checks if passed, return False if failed
        if len(path['path']) <= 1:
            value = str(path['value'])
            if value == 'null':
                value = 'None'
            entry_index = path['path'][0]
            try:
                entry[entry_index]
            except KeyError:
                print('Entry does not contain key: ', entry_index)
                return False
            try:
                regex_value = re.compile(value, re.IGNORECASE)
                if not re.search(regex_value, str(entry[entry_index]).encode('utf-8')):
                    return False
            except:
                print('ERR - regex compile: ', value)
                return False

        #sort pathes into groups based on first value
        else:
            found = False
            # new_path = {'value': path['value'], 'path': path['path'][1:]}
            for group in path_groups:
                if path['path'][0] == group[0]['path'][0]:
                    # print "found path: ", path
                    group.append(path)
                    found = True
                    break
            if not found:
                # print "didn't find path: ", path
                path_groups.append([path])

    #run each group together
    if not first:
        for group in path_groups:
            current_index = group[0]['path'][0]
            for i, path in enumerate(group):
                group[i] = {'value': path['value'], 'path': path['path'][1:]}
            #check if list, if so run against all
            if isinstance(entry, list):
                match = False
                for item in entry:
                    if single_filter(item, group, False):
                        match = True
                if not match:
                    return False
            else:
                if current_index in list(entry):
                    if not single_filter(entry[current_index], group, False):
                        return False
                else:
                    return False
        return True
    else:
        for group in path_groups:
            for i, path in enumerate(group):
                group[i] = {'value': path['value'], 'path': path['path'][1:]}
            if single_filter(entry, group, False):
                return True

def filter_data(data, search_input, max_results=5000):
    '''
    filter data based on path list or search dict arrays
    '''
    if isinstance(search_input, dict):
        print("dict found")
        search_input = [search_input]
    print(len(data))
    print("Running Search...")
    filtered_results = []
    for i, search_list in enumerate(search_input):
        if isinstance(search_list, dict):
            search_list = dict_to_paths(search_list, [], [])
        elif isinstance(search_list, list):
            search_list = search_list
        else:
            print("Invalid input type.  Must be 'dicts' or 'lists'")
            sys.exit(1)

        for search in search_list:
            print(search_list)
            try:
                int(search['path'][0])
            except:
                search['path'] = [i] + search['path']

        #Remove paths where value is None
        temp_paths = []
        for path in search_list:
            if path['value'] != str(None):
                temp_paths.append(path)
        search_list = temp_paths
        for entry in data:
            if len(filtered_results) >= max_results:
                break
            elif single_filter(entry, search_list, True) and entry not in filtered_results:
                filtered_results.append(entry)
    print("Results found: ", len(filtered_results))

    return filtered_results

def get_data(url=None, file_name='output.json', source="file"):
    '''
    Load data from either a source file or the api
    '''
    api_key = os.environ['PCT_API_READ_KEY']
    headers = {'x-api-key': api_key, 'Cache-Control': 'no-cache'}

    if source == "file":
        try:
            print("Loading File...")
            with open(file_name, 'rb') as stored_file:
                data = stored_file.read()
                return json.loads(data)
        except:
            print("File could not be loaded.  Pulling from API")
    elif source == "api":
        if not isinstance(url, str):
            print("You must use a url string when calling the api_to_file function.  Ex: parski.api_to_file(url='api_address.google.com')")
            print("Attempting to load from file...")
            if not os.path.isfile(file_name):
                print("%s cannot be found.  Exiting..." % file_name)
                sys.exit(1)
            else:
                try:
                    with open(file_name, 'rb') as stored_file:
                        data = stored_file.read()
                        return json.loads(data)
                except:
                    print("File could not be loaded.  Exiting...")
                    sys.exit(1)
        else:
            pass
    elif len(api_key) < 2:
        print("PCT_API_READ_KEY environment variable not set or set improperly. Exiting...")
        sys.exit(1)
    else:
        print("Invalid source parameter.  Exiting...")
        sys.exit(1)

    #LOAD FROM API

    tries = 1
    while tries <= 5:

        try:
            print("Sending api request...")
            req = requests.get(url, headers=headers)
            if req.status_code != 200:
                print ("WARNING:  API call status code was %s" % req.status_code)
            print("Request Returned!  Writing file....")
            json_string = json.dumps(req.json())
            clean_json_string = '[' + json_string.lstrip('[').rstrip(']') + ']'
            with open(file_name, 'wb') as output_file:
                output_file.write(clean_json_string)
            print("File written successfully...")
            break
        except Exception as error:
            #Check Errors
            try:
                req
            except:
                print("API Server could not be reached")
                print(error)
                sys.exit(1)
            print(error)
            print ("API request failed with a %s status code" % req.status_code)
            if req.status_code == 401:
                print ("Unauthorized.  Your API key isn't working...")
                break
            elif req.status_code == 404:
                print ("Not found.  Your url is probably wrong or you're not on VPN")
                break
            elif req.status_code == 502:
                print ("The response failed.  This happens from time to time...")
            elif req.status_code == 503:
                print ("Internal Server Error.  The API might be down...")
            else:
                print(error)

            #Retry
            if tries >= 4:
                print('4 api requests tried.  4 failed.  Exiting...')
                sys.exit(1)
            else:
                print("Trying again in %s seconds\n" % ((tries)**2))
                time.sleep((tries)**2)
                print("LET'S DO THIS!!!!")
            tries = tries + 1

    return json.loads(clean_json_string)

